/*****************************************************************************
 *------------------------ Utility  -----------------------------------------*
 *****************************************************************************/
#include <float.h>
#include "rscore.h"
#include "rsznl.h"

#if defined (_CVI_)
    #include <utility.h>
#endif

#pragma warning( disable : 4152 )

/*****************************************************************************
 *- Callback Declarations ---------------------------------------------------*
 *****************************************************************************/

/*****************************************************************************
 *- Common Callback Declarations --------------------------------------------*
 *****************************************************************************/

/*****************************************************************************
 * Function: rsznl_InitAttributes
 * Purpose:  This function inits attributes to the desired values if needed.
 *****************************************************************************/
ViStatus rsznl_InitAttributes(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    // - Driver Identification
    checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_DESCRIPTION, 0, RSZNL_SPECIFIC_DRIVER_DESCRIPTION));
    checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_PREFIX, 0, RSZNL_SPECIFIC_DRIVER_PREFIX));
    checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_LOCATOR, 0, RSZNL_SPECIFIC_DRIVER_LOCATOR));
    checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_VENDOR, 0, RSZNL_SPECIFIC_DRIVER_VENDOR));

    checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION, 0, RSZNL_CLASS_SPEC_MAJOR_VERSION));
    checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION, 0, RSZNL_CLASS_SPEC_MINOR_VERSION));

    // - Driver Capabilities
    checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_SUPPORTED_INSTRUMENT_MODELS, 0, RSZNL_SUPPORTED_INSTRUMENT_MODELS));
    checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_GROUP_CAPABILITIES, 0, RSZNL_GROUP_CAPABILITIES));
    checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_FUNCTION_CAPABILITIES, 0, RSZNL_FUNCTION_CAPABILITIES));

    // - Version Info
    checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_MAJOR_VERSION, 0, RSZNL_MAJOR_VERSION));
    checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_MINOR_VERSION, 0, RSZNL_MINOR_VERSION));
    checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_MINOR_MINOR_VERSION, 0, RSZNL_MINOR_MINOR_VERSION));

    checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_CLASS_DRIVER_MAJOR_VERSION, 0, RSZNL_CLASS_SPEC_MAJOR_VERSION));
    checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_CLASS_DRIVER_MINOR_VERSION, 0, RSZNL_CLASS_SPEC_MINOR_VERSION));

    checkErr(RsCore_SetSpecificDriverRevision(instrSession, RS_ATTR_SPECIFIC_DRIVER_REVISION));

Error:
    return error;
}

/*****************************************************************************
 * Function: rsznl_DefaultInstrSetup
 * Purpose:  This function sends a default setup to the instrument. The
 *           rsznl_reset function calls this function. The
 *           rsznl_RsInit function calls this function when the
 *           user passes VI_FALSE for the reset parameter. This function is
 *           useful for configuring settings that other instrument driver
 *           functions require.
 *
 *           Note:  Call this function only when the session is locked.
 *****************************************************************************/
ViStatus rsznl_DefaultInstrSetup(ViSession instrSession)
{
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));
    // Set all the attributes to the default state. Do not update inherent attributes!
    checkErr(RsCore_ApplyAttributeDefaults(instrSession, VI_FALSE));

    checkErr(rsznl_InitAttributes(instrSession));
    rsSession->fastSweepInstrument = 0;
    checkErr(RsCore_ResetRegistersEseSre(instrSession));

    if (!RsCore_Simulating(instrSession))
    {
        checkErr(RsCore_Write(instrSession, "*CLS"));
    }

    checkErr(rsznl_SynchronizeDynamicRepeatedCapability(instrSession, RSZNL_VAL_ALL_REP_CAPS));

Error:
    return error;
}

/*****************************************************************************
 * Function: rsznl_CheckStatus
 * Purpose:  This function checks the status of the instrument to detect
 *           whether the instrument has encountered an error. This function
 *           is called at the end of most exported driver functions. If the
 *           instrument reports an error, this function returns
 *           RS_ERROR_INSTRUMENT_SPECIFIC. The user can set the
 *           rsznl_ATTR_QUERY_INSTRUMENT_STATUS attribute to VI_FALSE to disable this
 *           check and increase execution speed.
 *
 *           Note:  Call this function only when the session is locked.
 *****************************************************************************/
ViStatus rsznl_CheckStatus(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    if (RsCore_QueryInstrStatus(instrSession) && !RsCore_Simulating(instrSession))
    {
        checkErr(RsCore_CheckStatus(instrSession, VI_SUCCESS));
    }

Error:
    return error;
}

/*****************************************************************************
 * Function: Get ViReal64 Array Window
*****************************************************************************/
ViStatus rsznl_getViReal64ArrayWindow(ViSession     instrSession,
                                      ViConstString cmdFormatString,
                                      ViInt32       window,
                                      ViReal64      array[],
                                      ViInt32       bufferSize,
                                      ViStatus      parameterErrorCode)
{  
    ViChar cmd[RS_MAX_MESSAGE_BUF_SIZE];

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, cmdFormatString, window);  // NOLINT(clang-diagnostic-format-nonliteral)

    return rsznl_getViReal64Array(instrSession, cmd, array, bufferSize, parameterErrorCode);
}

/*****************************************************************************
 * Function: Get ViReal64 Array
*****************************************************************************/
ViStatus rsznl_getViReal64Array(ViSession     instrSession,
                                ViConstString cmd,
                                ViReal64      array[],
                                ViInt32       bufferSize,
                                ViStatus      parameterErrorCode)
{  
    ViStatus error = VI_SUCCESS;

    if (array == NULL)
        return parameterErrorCode;

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, bufferSize, array, NULL));
    checkErr(rsznl_CheckStatus(instrSession));

Error:
    return error;
}

/*****************************************************************************
 * Function: Read Trace Data
 * Purpose:  This function reads out trace data from the instrument.
 *           It checks if data are ASCII or binary
*****************************************************************************/
ViStatus rsznl_dataReadTrace(ViSession         instrSession,
                             ViString          command,
                             ViInt32           arrayLength,
                             /*@out@*/ViReal64 traceData[],
                             ViPInt32          noofPoints)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));  

    //snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM ASC;:%s", command);
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:%s", command);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arrayLength, traceData, noofPoints));
    checkErr(RsCore_Write(instrSession, ":FORM ASC"));

Error:
    return error;
}

/*****************************************************************************
 * Function: Set OPC Timeout
 * Purpose:  Sets new OPC timeout
*****************************************************************************/
ViStatus rsznl_SetOPCTimeout(ViSession instrSession,
                             ViInt32   timeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_SetAttributeViInt32(instrSession, NULL, RS_ATTR_OPC_TIMEOUT, 0, timeout));

Error:
    return error;
}

/*****************************************************************************
 * Function: Get OPC Timeout
 * Purpose:  Gets new OPC timeout
*****************************************************************************/
ViStatus rsznl_GetOPCTimeout(ViSession instrSession,
                             ViInt32   *timeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_GetAttributeViInt32(instrSession, NULL, RS_ATTR_OPC_TIMEOUT, 0, timeout));

Error:
    return error;
}

/// HIFN Function used by Hi-level functions to obtain a repCap name from:
/// HIFN - attributeID
/// HIFN - repCapNameID
/// HIFN - 0-based index in the repCapNames
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or
/// HIPAR instrSession/rsvna_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR repCapNameId/RepCap Name ID.
/// HIPAR index/Zero based index in the repCapNames.
/// HIPAR bufferSize/Pass the number of bytes in the string you allocate for the repCapName.
/// HIPAR repCapName/String returning RepCap name
ViStatus rsznl_GetAttributeRepCapName(ViSession instrSession,
                                      ViUInt32  attributeId,
                                      ViString  repCapNameId,
                                      ViInt32   index,
                                      ViInt32   bufferSize,
                                      ViChar    repCapName[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, attributeId, repCapNameId, index, bufferSize, repCapName));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Write Memory Trace Data Extended
 * Purpose:  This function reads the stimulus values of the active data or
 *           memory trace.
 *****************************************************************************/
ViStatus rsznl_WriteMemoryTraceDataExt(ViSession instrSession,
                                       ViInt32   channel_Trace,
                                       ViInt32   dataFormat,
                                       ViInt32   no_ofPoints,
                                       ViReal64  traceData[])
{  
   ViStatus         error = VI_SUCCESS;
   RsCoreSessionPtr rsSession;
   ViChar           cmd[RS_MAX_MESSAGE_BUF_SIZE];
   ViInt32          dataSize = 0;
   ViByte*          data = NULL;
   ViString         traceFormatArr[] = { "SDAT","FDAT","MDAT","TSD",NULL };

   checkErr(RsCore_GetRsSession(instrSession, &rsSession));

   viCheckParm(RsCore_InvalidViInt32Range(instrSession, no_ofPoints, 1, 250000),
           4, "No of Points");

   if (rsSession->sessionType == RS_INTF_ASRL)
   {
       snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORMAT ASCII;:CALC%ld:DATA %s,", channel_Trace, traceFormatArr[dataFormat]);
       checkErr(RsCore_WriteAsciiViReal64Array(instrSession, cmd, traceData, no_ofPoints));
   }
   else
   {
       if (rsSession->sessionType == RS_INTF_GPIB)
       {
           checkErr(RsCore_Write(instrSession, ":SYST:COMM:GPIB:SELF:RTER EOI"));
       }

       snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:CALC%ld:DATA %s,", channel_Trace, traceFormatArr[dataFormat]);
       checkErr(RsCore_Convert_Float32Array_To_BinaryDataBlock(instrSession, traceData, no_ofPoints, &data, &dataSize, VI_FALSE));
       checkErr(RsCore_WriteBinaryDataBlock(instrSession, cmd, data, dataSize));

       if (rsSession->sessionType == RS_INTF_GPIB)
       {
           checkErr(RsCore_Write(instrSession, ":SYST:COMM:GPIB:SELF:RTER LFEOI"));
       }
   }

   checkErr(rsznl_CheckStatus(instrSession));

Error:
   if (data)
       free(data);

   RsCore_Write(instrSession, "FORM ASC");

   return error;
}

/*****************************************************************************
 * Function: Format Parameter Command
 * Purpose:  This function select appropriate command according to whether the
 *           trace already exists or not
 *****************************************************************************/
ViStatus rsznl_traceCommandSelect(ViSession instrSession,
                                  ViInt32   channel,
                                  ViString  traceName,
                                  ViChar    buffer[],
                                  ViInt32   *offset)
{  
    ViChar   rd_buffer [RS_MAX_MESSAGE_BUF_SIZE];
    ViStatus error;

    checkErr(rsznl_TraceList(instrSession, channel, rd_buffer, RS_MAX_MESSAGE_BUF_SIZE));

    if (strstr (rd_buffer, traceName))
        *offset = snprintf (buffer, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:PAR:MEAS ", channel);
    else
        *offset = snprintf (buffer, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:PAR:SDEF ", channel);

Error:
    return error;
}

/*****************************************************************************
 * Function: Send Touchstone File To Instrument
 * Purpose:  This function sends the content of a touchstone file to
 *           intrument
*****************************************************************************/
ViStatus rsznl_SendTouchstoneFileToInstrument(ViSession instrSession,
                                                       ViString  cmd,
                                                       ViString  touchstoneFile)
{  
    ViStatus error = VI_SUCCESS;

    viCheckParm(RsCore_InvalidNullPointer(instrSession, touchstoneFile), 2, "TouchstoneFile");

    checkErr(RsCore_WriteBinaryDataFromFile(instrSession, cmd, touchstoneFile));

    checkErr(rsznl_CheckStatus(instrSession));

Error:
    return error;
}

ViStatus rsznl_utility_error_message(ViSession instrSession,
                                     ViStatus  errorCode,
                                     ViChar    errorMessage[])
{
    ViStatus error = VI_SUCCESS;
    static   RsCoreStringValueTable errorTable =
             {
             RSZNL_ERROR_CODES_AND_MSGS,
    {0,      NULL}
             };

    (void)(RsCore_LockSession(instrSession));

    // all VISA and RS error codes are handled as well as codes in the table
    viCheckParm(RsCore_InvalidNullPointer(instrSession, errorMessage), 3, "Null address for Error Message");

    checkErr(RsCore_GetSpecificDriverStatusDesc(instrSession, errorCode, errorMessage, 256, errorTable));

Error:  
    (void)RsCore_UnlockSession(instrSession);
    return error;
}
